package com.ikanow.infinit.e.data_model.custom;

import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.bson.BSONObject;

import com.mongodb.DBCursor;
import com.mongodb.MongoException;

public class InfiniteMongoRecordReader extends RecordReader<Object, BSONObject>
{
	public InfiniteMongoRecordReader( InfiniteMongoInputSplit split )
	{
        _split = split;
        _cursor = split.getCursor();
    }

    @Override
    public void close(){
        if ( _cursor != null )
            _cursor.close();
    }

    @Override
    public Object getCurrentKey(){
        return _current.get( _split.getKeyField() );
    }

    @Override
    public BSONObject getCurrentValue(){
        return _current;
    }

    public float getProgress(){
        try {
            if ( _cursor.hasNext() ){
                return 0.0f;
            }
            else{
                return 1.0f;
            }
        }
        catch ( MongoException e ) {
            return 1.0f;
        }
    }

    @Override
    public void initialize( InputSplit split, TaskAttemptContext context ){
        _total = 1.0f;
    }

    @Override
    public boolean nextKeyValue(){
        try {
            if ( !_cursor.hasNext() )
                return false;

            _current = _cursor.next();
            _seen++;

            return true;
        }
        catch ( MongoException e ) {
            return false;
        }
    }


    private BSONObject _current;
    private final InfiniteMongoInputSplit _split;
    private final DBCursor _cursor;
    private float _seen = 0;
    private float _total;
}
